@@ -10,7 +10,6 @@
 2021/10/15 10:44   wangcc     1.0         None
 '''
 
-
 import json
 from .config import data_path, cal_config
 from joblib import Parallel, delayed
@@ -26,6 +25,7 @@ from pathlib import Path
 from .tools import bisect_left, bisect_right, to_intdate
 from pyarrow.dataset import field as ffield
 from pyarrow import dataset as ds
+
 markets = ["zz500", "hs300", "zz800", "zz1000", "all"]
 
 
@@ -59,6 +59,15 @@ class BaseProvider:
         """
         return list_features(freq)
 
+    def feature_info(self, field, freq):
+        """
+        return the config of the field
+        :param field: features str
+        :param freq: includdes Aday for natural day; Tday for trading day; report_period for financial statement
+        :return:
+        """
+        return FeatureD.feature_info(field, freq)
+
     def features(self, instruments, fields, start_time=None, end_time=None, freq="Tday", return_mat=False):
         """
         :param instruments: instruments list or market (ei. "zz500", "hs300", "zz800", "zz1000", "all"
@@ -150,7 +159,7 @@ class BaseProvider:
         if isinstance(fields, str):
             fields = [fields, ]
         if fields is None:
-            _, _, others, _, _= DatasetD.dataset_info(dataset)
+            _, _, others, _, _ = DatasetD.dataset_info(dataset)
             fields = list(others.keys())
         fields = list(fields)
         return DatasetD.datasets_fl(filt, fields, dataset, **kwargs)
@@ -370,6 +379,11 @@ class FeatureProvider(BaseDataProvider):
     def _uri_data(self):
         return os.path.join(data_path, "features", "{}", "{}.h5")
 
+    @property
+    def _uri_cfg(self):
+        """Static feature file uri."""
+        return os.path.join(data_path, "features", "{}", "{}_config.json")
+
     def features(self, instruments, fields, start_time=None, end_time=None, freq="Tday", return_mat=False):
         instruments_d = self.get_instruments_d(instruments, start_time, end_time, freq)
         column_names = self.get_column_names(fields)
@@ -378,9 +392,14 @@ class FeatureProvider(BaseDataProvider):
             return pd.DataFrame(columns=column_names)
         start_time = cal[0]
         end_time = cal[-1]
-        data = self.feature_processor(self._uri_data, instruments_d, column_names, start_time, end_time, freq, return_mat)
+        data = self.feature_processor(self._uri_data, instruments_d, column_names, start_time, end_time, freq,
+                                      return_mat)
         return data
 
+    def feature_info(self, field, freq):
+        config_path = self._uri_cfg.format(freq, field)
+        return FeatureD.read_config(config_path)
+
     @staticmethod
     def feature_processor(fpath, instruments_d, column_names, start_time, end_time, freq, return_mat=False):
         """
@@ -393,7 +412,8 @@ class FeatureProvider(BaseDataProvider):
             instruments_d = list(instruments_d.keys())
         start_index, end_index = Cal.locate_index(start_time, end_time, freq)
         dtname = cal_config[freq]["col_name"]
-        timeindex, mask, codes = read_mask(fpath.format(freq, dtname), instruments_d, start_index, end_index, return_mat)
+        timeindex, mask, codes = read_mask(fpath.format(freq, dtname), instruments_d, start_index, end_index,
+                                           return_mat)
         if mask is None:
             return pd.DataFrame(columns=column_names)
 
@@ -430,6 +450,15 @@ class FeatureProvider(BaseDataProvider):
         print(time.perf_counter() - start)
         return data
 
+    @staticmethod
+    def read_config(config_path):
+        if not os.path.exists(config_path):
+            print("No config fund")
+            return {}
+        with open(config_path, "r") as f:
+            json_cfg = json.load(f)
+        return json_cfg
+
 
 class DatasetProvider(BaseDataProvider):
     """Dataset provider class: Provide Dataset data."""
@@ -474,13 +503,13 @@ class DatasetProvider(BaseDataProvider):
         normalize_column_names = DatasetD.normalize_cache_fields(column_names)
         config_path = fpath + "_config.json"
         code_flt, time_flt, _, is_annrepo, format_ = DatasetD.read_config(config_path)
-        primary_keys = [code_flt, time_flt]            # 插入primary key
+        primary_keys = [code_flt, time_flt]  # 插入primary key
         if is_annrepo:
             primary_keys += ["report_period"]
-        normalize_column_names = primary_keys + normalize_column_names
+        normalize_column_names = normalize_column_names = DatasetD.normalize_cache_fields(primary_keys + normalize_column_names)
         code_flt = ffield(code_flt) if code_flt is not None else None
         time_flt = ffield(time_flt) if time_flt is not None else None
-        dset = ds.dataset(fpath, format=format_)                        # 还差不规则特征
+        dset = ds.dataset(fpath, format=format_)  # 还差不规则特征
 
         if code_flt is None:
             if time_flt is None:
@@ -501,16 +530,15 @@ class DatasetProvider(BaseDataProvider):
     def dataset_processor_fl(fpath, filt, column_names, **kwargs):
         """
         :param fpath:
-        :param filt:
+        :param filt: str or instance of pyarrow.dataset.field
         :param column_names:
         :param kwargs:
         :return:
         """
-
         # One process for one task, so that the memory will be freed quicker.
         config_path = fpath + "_config.json"
         code_flt, time_flt, _, is_annrepo, format_ = DatasetD.read_config(config_path)
-        primary_keys = [code_flt, time_flt]            # 插入primary key
+        primary_keys = [code_flt, time_flt]  # 插入primary key
         if is_annrepo:
             primary_keys += ["report_period"]
         column_names = primary_keys + column_names
@@ -524,7 +552,8 @@ class DatasetProvider(BaseDataProvider):
         else:
             for k in kwargs.keys():
                 exec(f"{k}=kwargs['{k}']")
-            filt=eval(parse_field(filt))
+            if isinstance(filt, str):
+                filt = eval(parse_field(filt))
             data = dset.to_table(columns=normalize_column_names, filter=filt).to_pandas()
         print(time.perf_counter() - start)
         return data
@@ -583,21 +612,17 @@ def read_mask(path, ori_instruments, start_index, end_index, mat_mode=False):
     instlist, instloc, rsilist, reilist, cidx = read_index(path)
     timeindex = read_mmap(path, "data")
     mask, codes = None, []
-    if ori_instruments is None:
-        codes = ori_instruments
-        infoloc = np.arange(len(instloc))
-    else:
-        codes, infoloc = get_availables(ori_instruments, instlist)
+    codes, infoloc = get_availables(ori_instruments, instlist)
     if len(infoloc) > 0:
         mask, amounts = gen_idxs(cidx, infoloc, instloc, rsilist, reilist, start_index, end_index)
-        if not mat_mode:                             # 返回矩阵形式
+        if not mat_mode:  # 返回矩阵形式
             codes = np.repeat(codes, amounts)
             timeindex = timeindex[mask]
     return timeindex, mask, codes
 
 
 def get_availables(instruments, availables):
-    series = pd.Series(np.arange(len(instruments)), index=instruments).reindex(availables).dropna()
+    series = pd.Series(np.arange(len(availables)), index=availables).reindex(instruments).dropna()
     return series.index.values, series.values.astype(int)
 
 
@@ -615,7 +640,7 @@ def gen_idxs(cidx, infoloc, instloc, rsilist, reilist, start_index, end_index):
         h5si, h5ei = tsi + instsiloc, tei + instsiloc
         amounts.append(h5ei - h5si + 1)
         mask[h5si:h5ei + 1] = True
-        assert amounts[-1]>0
+        assert amounts[-1] > 0
     return mask, amounts
 
 
